<?php

/**
 * @file
 * Theme for Gantt views.
 */

use Drupal\Core\Url;

/**
 * Template preprocess views gantt.
 *
 * @param array $variables
 *   Array variable.
 */
function template_preprocess_views_view_gantt(array &$variables) {
  $view = $variables['view'];
  $style = $view->style_plugin;
  $setting = $style->options;
  $setting['format_date'] = 'd/m/Y';
  $variables["view_id"] = $view->storage->id();
  $baseTable = $view->storage->get('base_table');
  $variables["display_id"] = $variables["view"]->current_display;
  $entity_type_id = 'node';
  $field_name = 'test';
  $entityId = 1;
  $handler = 'x';
  $colors = [
    'primary',
    'warning',
    'success',
    'danger',
    'info',
    'dark',
    'secondary',
  ];

  if ($setting["cdn"]) {
    $variables['view']->element['#attached']['library'][] = 'gantt/gantt.cdn';
  }
  else {
    $variables['view']->element['#attached']['library'][] = 'gantt/gantt';
  }
  $id = implode('-', ["gantt", $view->id(), $view->current_display]);
  $variables["id_gantt"] = $id;
  $variables['view']->element['#attached']['drupalSettings']['gantt'][$id] = [
    'date_format' => str_replace(
      ['Y', 'm', 'd', 'H', 'i', 's'],
      ['%Y', '%m', '%d', '%H', '%i', '%s'],
      $setting['format_date']
    ),
    'id' => $variables["id_gantt"],
    'owner' => $setting['owner'],
    'ajax' => Url::fromRoute('gantt.ajax', [
      'entity_type' => $entity_type_id,
      'entity_field' => $field_name,
      'entity_id' => $entityId,
      'type' => $handler,
    ])->toString(),
    'import' => Url::fromRoute('gantt.import', [
      'entity_type' => $entity_type_id,
      'entity_field' => $field_name,
      'entity_id' => $entityId,
      'type' => $handler,
    ])->toString(),
  ];
  $columns = [];

  // Dialog width.
  $variables["data"] = convert_gantt_data($variables['rows'], $setting);
  $variables["owner"] = [];
  $variables["add"] = '';
}

/**
 * {@inheritDoc}
 */
function convert_gantt_data($items, array $setting) {
  $links = [];
  $outData = [];
  $mappingGantt = [
    'text',
    'type',
    'start_date',
    'duration',
    'progress',
    'open',
    'parent',
    'owner',
  ];
  $fieldDateType = 'date';

  foreach ($items as $delta => $item) {
    $entity = $item->_entity;
    $fieldDefinitions = $entity->getFieldDefinitions();
    if (!empty($fieldDefinitions[$setting['start_date']])) {
      $dateSettings = $fieldDefinitions[$setting['start_date']]->getSettings();
      $fieldDateType = $fieldDefinitions[$setting['start_date']]->getType();
    }
    $data[$delta] = ['id' => $entity->id()];
    foreach ($mappingGantt as $ganttField) {
      if (!empty($field_name = $setting[$ganttField])) {
        $value = $entity->get($field_name)->getString();
        if (!empty($value)) {
          $data[$delta][$ganttField] = $value;
        }
        if ($ganttField == 'start_date') {
          if (!empty($entity->get($field_name)->date)) {
            $data[$delta][$ganttField] = $entity->get($field_name)->date->format($setting['format_date']);
          }
          if ($fieldDateType == 'daterange' && $dateValue = $entity->get($field_name)->value) {
            // $endValue = $entity->get($field_name)->end_value;
            $data[$delta][$ganttField] = \Drupal::service('date.formatter')
              ->format(strtotime($dateValue), 'short');
          }
        }

        // Process owner.
        if ($ganttField == 'owner' && !$entity->get($field_name)->isEmpty()) {
          $values = $entity->get($field_name)->getValue();
          $data[$delta][$ganttField] = [];
          foreach ($values as $value) {
            if (!empty($value->entity)) {
              $data[$delta][$ganttField][] = $value->entity->id();
            }
          }
          // @todo remove later for multiuser.
          $data[$delta][$ganttField] = $value->entity->id();
        }
      }
    }
    // Process Links.
    if (!empty($setting['links'])) {
      $valueLinks = $entity->get($setting['links'])->getValue();
      foreach ($valueLinks as $valueLink) {
        $links[] = [
          'id' => implode('-', [$entity->id(), $valueLink['first']]),
          'source' => $entity->id(),
          'target' => (int) $valueLink['first'],
          'type' => (int) $valueLink['second'],
        ];
      }
    }

  }
  if (!empty($data)) {
    $outData['data'] = $data;
    if (!empty($links)) {
      $outData['links'] = $links;
    }
  }
  return $outData;
}
